@@ -153,8 +153,8 @@ mch_write(char_u *p, int len)
 mch_inchar(
     char_u  *buf,
     int	    maxlen,
-    long    time,		// milli seconds
-    int	    tb_change_cnt)
+    long    time,		// milliseconds
+    int	    tb_change_cnt UNUSED)
 {
     int	    len;
     long    utime;
@@ -571,7 +571,7 @@ mch_input_isatty(void)
     void
 fname_case(
     char_u	*name,
-    int		len)		// buffer size, ignored here
+    int		len UNUSED)		// buffer size, ignored here
 {
     struct FileInfoBlock    *fib;
     size_t		    flen;
@@ -623,7 +623,6 @@ get_fib(char_u *fname)
     return fib;
 }
 
-#ifdef FEAT_TITLE
 /*
  * set the title of our window
  * icon name is not set
@@ -660,7 +659,6 @@ mch_can_restore_icon(void)
 {
     return FALSE;
 }
-#endif
 
     void
 mch_setmouse(int on UNUSED)
@@ -839,7 +837,7 @@ mch_setperm(char_u *name, long perm)
  * Set hidden flag for "name".
  */
     void
-mch_hide(char_u *name)
+mch_hide(char_u *name UNUSED)
 {
     // can't hide a file
 }
@@ -891,10 +889,50 @@ mch_mkdir(char_u *name)
  * Return -1 if unknown.
  */
     int
-mch_can_exe(char_u *name, char_u **path, int use_path)
+mch_can_exe(char_u *name, char_u **path UNUSED, int use_path)
 {
-    // TODO
-    return -1;
+    int exe = -1;
+#ifdef __amigaos4__
+    // Load file sections using elf.library or hunk.library.
+    BPTR seg = LoadSeg(name);
+
+    if (seg && GetSegListInfoTags(seg, GSLI_Native, NULL, TAG_DONE) !=
+        GetSegListInfoTags(seg, GSLI_68KHUNK, NULL, TAG_DONE))
+    {
+        // Test if file permissions allow execution.
+        struct ExamineData *exd = ExamineObjectTags(EX_StringNameInput, name);
+
+        exe = (exd && !(exd->Protection & EXDF_NO_EXECUTE)) ? 1 : 0;
+        FreeDosObject(DOS_EXAMINEDATA, exd);
+    }
+    else
+    {
+        exe = 0;
+    }
+
+    UnLoadSeg(seg);
+
+    // Search for executable in path if applicable.
+    if (!exe && use_path)
+    {
+        // Save current working dir.
+        BPTR cwd = GetCurrentDir();
+        struct PathNode *head = DupCmdPathList(NULL);
+
+        // For each entry, recur to check for executable.
+        for(struct PathNode *tail = head; !exe && tail;
+            tail = (struct PathNode *) BADDR(tail->pn_Next))
+        {
+            SetCurrentDir(tail->pn_Lock);
+            exe = mch_can_exe(name, path, 0);
+        }
+
+        // Go back to where we were.
+        FreeCmdPathList(head);
+        SetCurrentDir(cwd);
+    }
+#endif
+    return exe;
 }
 
 /*
@@ -904,7 +942,7 @@ mch_can_exe(char_u *name, char_u **path, int use_path)
  * NODE_OTHER: non-writable things
  */
     int
-mch_nodetype(char_u *name)
+mch_nodetype(char_u *name UNUSED)
 {
     // TODO
     return NODE_NORMAL;
@@ -940,9 +978,7 @@ mch_exit(int r)
 	out_flush();
     }
 
-#ifdef FEAT_TITLE
     mch_restore_title(SAVE_RESTORE_BOTH);    // restore window title
-#endif
 
     ml_close_all(TRUE);		    // remove all memfiles
 
@@ -1260,7 +1296,7 @@ mch_call_shell(
     if (close_win)
     {
 	// if Vim opened a window: Executing a shell may cause crashes
-	emsg(_("E360: Cannot execute shell with -f option"));
+	emsg(_(e_cannot_execute_shell_with_f_option));
 	return -1;
     }
 
@@ -1442,9 +1478,7 @@ mch_call_shell(
 	cur_tmode = TMODE_UNKNOWN;
 	settmode(TMODE_RAW);		// set to raw mode
     }
-#ifdef FEAT_TITLE
     resettitle();
-#endif
     if (term_console)
 	win_resize_on();		// window resize events activated
     return retval;
@@ -1456,7 +1490,7 @@ mch_call_shell(
  * trouble with lattice-c programs.
  */
     void
-mch_breakcheck(int force)
+mch_breakcheck(int force UNUSED)
 {
    if (SetSignal(0L, (long)(SIGBREAKF_CTRL_C|SIGBREAKF_CTRL_D|SIGBREAKF_CTRL_E|SIGBREAKF_CTRL_F)) & SIGBREAKF_CTRL_C)
 	got_int = TRUE;
@@ -1721,7 +1755,7 @@ mch_getenv(char_u *var)
  */
 // ARGSUSED
     int
-mch_setenv(char *var, char *value, int x)
+mch_setenv(char *var, char *value, int x UNUSED)
 {
 #ifdef FEAT_ARP
     if (!dos2)
