@@ -375,7 +375,11 @@ static char *(features[]) =
 	"-mouse_dec",
 # endif
 # ifdef FEAT_MOUSE_GPM
+#  ifdef DYNAMIC_GPM
+	"+mouse_gpm/dyn",
+#  else
 	"+mouse_gpm",
+#  endif
 # else
 	"-mouse_gpm",
 # endif
@@ -586,11 +590,7 @@ static char *(features[]) =
 #if defined(USE_SYSTEM) && defined(UNIX)
 	"+system()",
 #endif
-#ifdef FEAT_TAG_BINS
 	"+tag_binary",
-#else
-	"-tag_binary",
-#endif
 	"-tag_old_static",
 	"-tag_any_white",
 #ifdef FEAT_TCL
@@ -750,6 +750,412 @@ static char *(features[]) =
 
 static int included_patches[] =
 {   /* Add new patch number below this line */
+/**/
+    4586,
+/**/
+    4585,
+/**/
+    4584,
+/**/
+    4583,
+/**/
+    4582,
+/**/
+    4581,
+/**/
+    4580,
+/**/
+    4579,
+/**/
+    4578,
+/**/
+    4577,
+/**/
+    4576,
+/**/
+    4575,
+/**/
+    4574,
+/**/
+    4573,
+/**/
+    4572,
+/**/
+    4571,
+/**/
+    4570,
+/**/
+    4569,
+/**/
+    4568,
+/**/
+    4567,
+/**/
+    4566,
+/**/
+    4565,
+/**/
+    4564,
+/**/
+    4563,
+/**/
+    4562,
+/**/
+    4561,
+/**/
+    4560,
+/**/
+    4559,
+/**/
+    4558,
+/**/
+    4557,
+/**/
+    4556,
+/**/
+    4555,
+/**/
+    4554,
+/**/
+    4553,
+/**/
+    4552,
+/**/
+    4551,
+/**/
+    4550,
+/**/
+    4549,
+/**/
+    4548,
+/**/
+    4547,
+/**/
+    4546,
+/**/
+    4545,
+/**/
+    4544,
+/**/
+    4543,
+/**/
+    4542,
+/**/
+    4541,
+/**/
+    4540,
+/**/
+    4539,
+/**/
+    4538,
+/**/
+    4537,
+/**/
+    4536,
+/**/
+    4535,
+/**/
+    4534,
+/**/
+    4533,
+/**/
+    4532,
+/**/
+    4531,
+/**/
+    4530,
+/**/
+    4529,
+/**/
+    4528,
+/**/
+    4527,
+/**/
+    4526,
+/**/
+    4525,
+/**/
+    4524,
+/**/
+    4523,
+/**/
+    4522,
+/**/
+    4521,
+/**/
+    4520,
+/**/
+    4519,
+/**/
+    4518,
+/**/
+    4517,
+/**/
+    4516,
+/**/
+    4515,
+/**/
+    4514,
+/**/
+    4513,
+/**/
+    4512,
+/**/
+    4511,
+/**/
+    4510,
+/**/
+    4509,
+/**/
+    4508,
+/**/
+    4507,
+/**/
+    4506,
+/**/
+    4505,
+/**/
+    4504,
+/**/
+    4503,
+/**/
+    4502,
+/**/
+    4501,
+/**/
+    4500,
+/**/
+    4499,
+/**/
+    4498,
+/**/
+    4497,
+/**/
+    4496,
+/**/
+    4495,
+/**/
+    4494,
+/**/
+    4493,
+/**/
+    4492,
+/**/
+    4491,
+/**/
+    4490,
+/**/
+    4489,
+/**/
+    4488,
+/**/
+    4487,
+/**/
+    4486,
+/**/
+    4485,
+/**/
+    4484,
+/**/
+    4483,
+/**/
+    4482,
+/**/
+    4481,
+/**/
+    4480,
+/**/
+    4479,
+/**/
+    4478,
+/**/
+    4477,
+/**/
+    4476,
+/**/
+    4475,
+/**/
+    4474,
+/**/
+    4473,
+/**/
+    4472,
+/**/
+    4471,
+/**/
+    4470,
+/**/
+    4469,
+/**/
+    4468,
+/**/
+    4467,
+/**/
+    4466,
+/**/
+    4465,
+/**/
+    4464,
+/**/
+    4463,
+/**/
+    4462,
+/**/
+    4461,
+/**/
+    4460,
+/**/
+    4459,
+/**/
+    4458,
+/**/
+    4457,
+/**/
+    4456,
+/**/
+    4455,
+/**/
+    4454,
+/**/
+    4453,
+/**/
+    4452,
+/**/
+    4451,
+/**/
+    4450,
+/**/
+    4449,
+/**/
+    4448,
+/**/
+    4447,
+/**/
+    4446,
+/**/
+    4445,
+/**/
+    4444,
+/**/
+    4443,
+/**/
+    4442,
+/**/
+    4441,
+/**/
+    4440,
+/**/
+    4439,
+/**/
+    4438,
+/**/
+    4437,
+/**/
+    4436,
+/**/
+    4435,
+/**/
+    4434,
+/**/
+    4433,
+/**/
+    4432,
+/**/
+    4431,
+/**/
+    4430,
+/**/
+    4429,
+/**/
+    4428,
+/**/
+    4427,
+/**/
+    4426,
+/**/
+    4425,
+/**/
+    4424,
+/**/
+    4423,
+/**/
+    4422,
+/**/
+    4421,
+/**/
+    4420,
+/**/
+    4419,
+/**/
+    4418,
+/**/
+    4417,
+/**/
+    4416,
+/**/
+    4415,
+/**/
+    4414,
+/**/
+    4413,
+/**/
+    4412,
+/**/
+    4411,
+/**/
+    4410,
+/**/
+    4409,
+/**/
+    4408,
+/**/
+    4407,
+/**/
+    4406,
+/**/
+    4405,
+/**/
+    4404,
+/**/
+    4403,
+/**/
+    4402,
+/**/
+    4401,
+/**/
+    4400,
+/**/
+    4399,
+/**/
+    4398,
+/**/
+    4397,
+/**/
+    4396,
+/**/
+    4395,
+/**/
+    4394,
+/**/
+    4393,
+/**/
+    4392,
+/**/
+    4391,
+/**/
+    4390,
+/**/
+    4389,
+/**/
+    4388,
+/**/
+    4387,
+/**/
+    4386,
+/**/
+    4385,
+/**/
+    4384,
 /**/
     4383,
 /**/
